# -*- coding: utf-8 -*-
import wsgi


class NotFound(Exception):
    pass

class Redirect(Exception):
    '''Deprecated'''
    def __init__(self, url, **kwargs):
        self.res = wsgi.redirect(url, **kwargs)

class PermanentRedirect(Exception):
    '''Deprecated'''
    def __init__(self, url, **kwargs):
        self.res = wsgi.redirect(url, permanent=True, **kwargs)

class BasicAuth(Exception):
    def __init__(self):
        self.res = wsgi.Response('Authorization Error', 
            code=401, content_type='text/plain',
            headers={'WWW-Authenticate' : 'Basic realm="Enter password"'},
        )



# -*- coding: utf-8 -*-
import sys
import datetime
import traceback

import pysi


WEEKDAYS = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
MONTHS = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 
    'Sep', 'Oct', 'Nov', 'Dec']


def obj_from_str(str_or_obj):
    '''
    Если передана строка, пытается проимпортировать модуль и вернуть объект.
    '''
    if isinstance(str_or_obj, basestring):
        if '.' not in str_or_obj:
            str_or_obj += '.'
        mod_name, obj_name = str_or_obj.rsplit('.', 1)
        __import__(mod_name)
        mod = sys.modules[mod_name]
        return getattr(mod, obj_name) if obj_name else mod
    else:
        return str_or_obj

def list_obj_from_str(lst):
    for i in xrange(len(lst)):
        obj = obj_from_str(lst[i])
        lst[i] = obj

class cache_property(object):
    def __init__(self, f):
        self.f = f

    def __get__(self, instance, owner):
        value = self.f(instance)
        setattr(instance, self.f.__name__, value)
        return value

def make_traceback(host):
    head = '%s | %s' % (host, datetime.datetime.now())
    line = '-' * len(head)
    return '\n%s\n%s\n%s\n%s' % (line, head, line,
        ''.join(traceback.format_exception(*sys.exc_info())[1:]))
            
def anticache_headers():
    expires = http_date(datetime.datetime.utcnow())
    return {
        'Expires' : expires,
        'Last-Modified' : expires,
        'Pragma' : 'no-cache',
        'Cache-Control' : 'private, no-cache, no-store, must-revalidate, max-age=0, pre-check=0, post-check=0',
    }

def http_date(dt):
    '''
    Дата обновления документа в http формате
    '''
    week = WEEKDAYS[dt.weekday()]
    month = MONTHS[dt.month - 1]
    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (week, dt.day, month,
        dt.year, dt.hour, dt.minute, dt.second)


# -*- coding: utf-8 -*-

from wsgi import App, Request, Response, redirect
from config import cfg
from exceptions import NotFound, Redirect, PermanentRedirect, BasicAuth
from routing import urls, add_apps, add_urls, url4, auto_routing
from decorators import as_response, as_text, as_html, as_json, to_template
from decorators import route, error, basic_auth, anticache
from template import render_to_string, render_to_response


# -*- coding: utf-8 -*-
import wsgi


class Cfg(object):
    DEBUG = False
    BASIC_AUTH_SSL_ONLY = True  # basic-авторизация только по https
    BASIC_AUTH_DB = {}
    INSTALLED_APPS = []
    CONTEXT_PROCESSORS = []
    REQUEST_MIDDLEWARES = []
    RESPONSE_MIDDLEWARES = []
    CHARSET = 'utf-8'
    POST_MAX_SIZE = 1024 * 1024 * 1  # максимальный размер пост-данных, None - не ограничивать
    POST_MAX_MEMFILE = 1024 * 100  # максимальный размер POST-данных, загружаемый в память. Если больше, создаётся временный файл.
    ERROR_PAGES = {
        404: lambda rq: wsgi.Response(wsgi.HTML_404, code=404, content_type='text/html'),
        500: lambda rq: wsgi.Response(wsgi.HTML_500, code=500, content_type='text/html'),
    }
    TEMPLATE_CACHE_SIZE = 250
    TEMPLATE_AUTO_RELOAD = False
    FLASH_COOKIE_NAME = u'pysi.flash'

    def set_defaults(self, **kwargs):
        '''
        Добавляем дефолтные атрибуты
        '''
        for k, v in kwargs.iteritems():
            if not hasattr(self, k) and k.isupper():
                setattr(self, k, v)

cfg = Cfg()



# -*- coding: utf-8 -*-
try:
    import simplejson as json
except ImportError:
    from django.utils import simplejson as json

from wsgi import Response
from exceptions import BasicAuth, Redirect
from routing import urls
from config import cfg
from util import anticache_headers
from template import render_to_response


def route(url, name=None):
    ''' Декоратор роутинга '''
    def wrapper(func):
        urls.add_rule(url, func, name)
        return func
    return wrapper

def as_response(dumper=None, **response_kwargs):
    ''' Базовый декоратор response '''
    def decorator(func):
        def wrapper(*args, **kwargs):
            res = func(*args, **kwargs)
            if not isinstance(res, Response):
                if dumper:
                    res = dumper(res)
                res = Response(res, **response_kwargs)
            return res
        return wrapper
    return decorator
    
def as_text(**kwargs):
    ''' Декоратор текстовый response '''
    return as_response(content_type='text/plain', **kwargs)
    
def as_html(**kwargs):
    ''' Декоратор html-response '''
    return as_response(content_type='text/html', **kwargs)

def as_json(**kwargs):
    ''' Декоратор json_response '''
    return as_response(dumper=json.dumps,
        content_type='application/json', **kwargs)

def to_template(fn, **response_kwargs):
    '''
    Декоратор шаблонизации
    '''
    def decorator(func):
        def wrapper(rq, *args, **kwargs):
            res = func(rq, *args, **kwargs)
            if isinstance(res, Response):
                return res
            return render_to_response(
                rq, fn, res, **response_kwargs)
        return wrapper
    return decorator

def anticache(func):
    ''' Декоратор браузерного антикэша '''
    def wrapper(*args, **kwargs):
        res = func(*args, **kwargs)
        res.headers.update(anticache_headers())
        return res
    return wrapper

def error(code):
    ''' Декоратор страницы ошибки '''
    def wrapper(func):
        cfg.ERROR_PAGES[code] = func
        return func
    return wrapper

def basic_auth(func):
    '''
    Basic-авторизация.
    '''
    def wrapper(rq, *args, **kwargs):
        rq.basic_user
        return func(rq, *args, **kwargs)
    return wrapper
